/*!
    \file
    \brief Declares the class GLWidget.
*/


#ifndef DEFIBERIZER_GLWIDGET_H
#define DEFIBERIZER_GLWIDGET_H

#include <QGLWidget>

class QImage;
class QMovie;
class QString;
class QWidget;

/*!
    \class GLWidget
    \brief Defines a widget for displaying images as OpenGL textures. Inherits QGLWidget.
*/
class GLWidget : public QGLWidget
{
    Q_OBJECT

    friend class MainView;

public:
    GLWidget(QWidget *parent = 0, const QString &qsImagePath = "");
    GLWidget(QWidget *parent = 0, const QImage  &qiImage = QImage());
    ~GLWidget();

    const QString getImagePath();
    const QImage  getImage();

public slots:
    void slotUpdate();
    void slotResize();

protected:
    void initializeGL();
    void resizeGL(int iWidth, int iHeight);
    void paintGL();
    void loadGLTexture();

    bool setImagePath   (const QString &qsImagePath);
    bool setImage       (const QImage  &qiImage);

protected slots:
    bool slotUpdateImage(const QString &qsImagePath);
    bool slotUpdateImage(const QImage  &qiImage);

private:
    // Display Data
    GLuint pbo_buffer;  // Front and back CA buffers
    struct cudaGraphicsResource *cuda_pbo_resource; // CUDA Graphics Resource (to transfer PBO)

    GLuint texid;       // Texture for display

    unsigned char *pixels;
    GLuint   m_pTexture;             //!< The index of the OpenGL texture.
    QString  m_pQSImagePath;         //!< The path to the image for the texture (if used).
    QImage   m_pQImage;              //!< The image that is used as a OpenGL texture.
    QMovie  *m_pQMovie;
};

#endif // DEFIBERIZER_GLWIDGET_H


